<?php

/**
 * The public-facing functionality of the plugin.
 * php version 7.2.34
 *
 * @category Pedalo_Plugin
 * @package  Pedalo_Connector
 * @subpackage Pedalo_Connector/public
 * @author   Pedalo <developers@pedalo.co.uk>
 * @link       www.pedalo.co.uk
 * @since      1.0.0
 */

/**
 * The public-facing functionality of the plugin.
 *
 * Defines the plugin name, version, and two examples hooks for how to
 * enqueue the public-facing stylesheet and JavaScript.
 *
 * @category Pedalo_Plugin
 * @package  Pedalo_Connector
 * @subpackage Pedalo_Connector/public
 * @author   Pedalo <developers@pedalo.co.uk>
 * @link       www.pedalo.co.uk
 * @since      1.0.0
 */
class Pedalo_connector_Public
{

    /**
     * The ID of this plugin.
     *
     * @since    1.0.0
     * @access   private
     * @var      string    $plugin_name    The ID of this plugin.
     */
    private $plugin_name;

    /**
     * The version of this plugin.
     *
     * @since    1.0.0
     * @access   private
     * @var      string    $version    The current version of this plugin.
     */
    private $version;

    /**
     * Initialize the class and set its properties.
     *
     * @since    1.0.0
	 *
     * @param      string    $plugin_name       The name of the plugin.
     *
	 * @param      string    $version    The version of this plugin.
     */
    public function __construct($plugin_name, $version)
    {

        $this->plugin_name = $plugin_name;
        $this->version = $version;

    }

    /**
     * Register the stylesheets for the public-facing side of the site.
     *
     * @since    1.0.0
	 * @return   null
     */
    public function enqueue_styles()
    {

        /**
         * This function is provided for demonstration purposes only.
         *
         * An instance of this class should be passed to the run() function
         * defined in Pedalo_connector_Loader as all of the hooks are defined
         * in that particular class.
         *
         * The Pedalo_connector_Loader will then create the relationship
         * between the defined hooks and the functions defined in this
         * class.
         */

        wp_enqueue_style($this->plugin_name, plugin_dir_url(__FILE__) .
            'css/pedalo_connector-public.css', array(), $this->version, 'all');

    }

    /**
     * Register the JavaScript for the public-facing side of the site.
     *
     * @since    1.0.0
	 * @return   null
     */
    public function enqueue_scripts()
    {

        /**
         * This function is provided for demonstration purposes only.
         *
         * An instance of this class should be passed to the run() function
         * defined in Pedalo_connector_Loader as all of the hooks are defined
         * in that particular class.
         *
         * The Pedalo_connector_Loader will then create the relationship
         * between the defined hooks and the functions defined in this
         * class.
         */

        wp_enqueue_script($this->plugin_name, plugin_dir_url(__FILE__) .
            'js/pedalo_connector-public.js', array('jquery'), $this->version, false);

    }

    /**
 	* Admin login : required to get plugin update informations
 	* 
 	* 
 	* @return null
	*/
    public function login_admin_user()
    {
        $api_key = '';
        if(isset($_SERVER['HTTP_PEDALO_API_KEY'])){
            $api_key = $_SERVER['HTTP_PEDALO_API_KEY'];
        }

        if($api_key != '' && isset($_GET['admin_login']) && $_GET['admin_login'] == 1){

            if ($api_key == get_option('pedalo_api_key')) {
                $user_id = 1;

                $user = get_user_by( 'id', $user_id );

                if($user->ID == ''){
                
                    $args = array(
                        'role__in'    => array('administrator'),
                        'orderby' => 'id',
                        'order'   => 'ASC',
                        );
                    $users = get_users($args);
                    $user = $users[0]->data;
                }              
                
                
                clean_user_cache($user->ID);
                wp_clear_auth_cookie();
                wp_set_current_user( $user->ID, $user->user_login );
                wp_set_auth_cookie( $user->ID, true, true );
                update_user_caches( $user );
                exit;
            }
        }
    }
	/**
 	* Register api route
 	* 
 	* 
 	* @return null
	*/
	public function registerRestRoute()
    {
		 register_rest_route(
            'pedalo/', 'plugin_session', array(
            'methods' => 'POST,GET',
            'callback' => array($this,'pedaloConnectorEndpoint'),
            )
        );
    }
	/**
 	* Pedalo connector plugin authentication and api result
 	* 
 	* @param array $wprequest gets plugins to be updated
 	* 
 	* @return Returns json for plugin information
 	*/
    public function pedaloConnectorEndpoint($wprequest)
    {
        $api_key = '';

        if(isset($_SERVER['HTTP_PEDALO_API_KEY'])){
            $api_key = $_SERVER['HTTP_PEDALO_API_KEY'];
        }

        if ($_SERVER['REQUEST_METHOD'] != "POST") {
            return new WP_Error(
                'not_valid_request',
                'Access denied', array('status' => 404)
            );
        }

        if ($api_key != get_option('pedalo_api_key')) {
            return new WP_Error(
                'not_valid_api_key',
                'Invalid api key', array('status' => 404)
            );
        }
		try {
        	$json = array();
        	$json = $this->getUpdates();
        	$updates = $this->update($wprequest, $json);
        	if ($updates) {
            	return $updates;
        	}
			return $json;
		}catch (Exception $e) {
			 error_log( $e->getMessage());
		}        
    }
	
	/**
 	* Get all updates available for plugins
 	* 
 	* 
 	* @return array 
	*/
    public function getUpdates()
    {
        $json = array();
        include ABSPATH . WPINC . '/version.php';
        include_once ABSPATH . 'wp-includes/update.php';

        // Checking for core updated

        $json['name'] = get_bloginfo('name');
        $json['url'] = get_bloginfo('wpurl');
        $json['timestamp'] = time();
        $json['php'] = phpversion();
        $json['ssl'] = is_ssl();

        $json['core']['version'] = $wp_version;

        wp_version_check();
        $update_core = get_site_transient('update_core');

        if ($wp_version == $update_core->updates[0]->version) {
            $json['core']['update'] = false;
        } else {
            $json['core']['update'] = $update_core->updates[0]->version;
        }

        // Checking for plugins updated
        if (!function_exists('get_plugins')) {
            include_once ABSPATH . 'wp-admin/includes/plugin.php';
        }
        delete_site_transient('update_plugins');
        $ch = curl_init();
        $headers = array(
            "Content-Type:application/json",
            "Accept:application/json; charset=UTF-8",
            "PEDALO-API-KEY:".$_SERVER['HTTP_PEDALO_API_KEY'],
            "Authorization:Basic cGVkYWxvOlBlZGFsbzEyMyE=",
        );
        curl_setopt($ch, CURLOPT_HTTPHEADER, $headers);
        curl_setopt($ch, CURLOPT_URL, str_ireplace("http:","https:",get_site_url()). '/?admin_login=1');
        curl_setopt($ch, CURLOPT_USERAGENT,'Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Ubuntu Chromium/32.0.1700.107 Chrome/32.0.1700.107 Safari/537.36');
        curl_setopt($ch, CURLOPT_POST, true);
        curl_setopt($ch, CURLOPT_POSTFIELDS, '{"updateversion":"admin_login"}');
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($ch, CURLOPT_COOKIESESSION, true);
        curl_setopt($ch, CURLOPT_COOKIEJAR, 'cookie-name');//could be empty, but cause problems on some hosts
        curl_setopt($ch, CURLOPT_COOKIEFILE, ABSPATH .'wp-content/plugins/ip4.x/file/tmp');//could be empty, but cause problems on some hosts
        curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, false);
        curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, false);
        $answer = curl_exec($ch);
        
        //another request preserving the session
        curl_setopt($ch, CURLOPT_URL, str_ireplace("http:","https:",get_site_url()). '/wp-admin/plugins.php');
        curl_setopt($ch, CURLOPT_POST, false);
        curl_setopt($ch, CURLOPT_POSTFIELDS, "");
        $answer = curl_exec($ch);  
        
        wp_update_plugins();
        
        $plugins = get_plugins();
        $update_plugins = get_site_transient('update_plugins');

        $response['plugins'] = $update_plugins->response;
        $response['no_update'] = $update_plugins->no_update;

        $active_plugins = get_option('active_plugins');
        foreach ($plugins as $key => $single_plugin) {
            $active = false;
            if (in_array($key, $active_plugins)) {
                $active = true;
            }
            $update = false;
            if (isset($response['no_update'][$key])) {
                $update = 'false';
            } else if (isset($response['plugins'][$key])) {
                $update = $response['plugins'][$key]->new_version;
            } else {
                //$update = 'Cannot retrieve update for this plugin';
                $update = 'false';
            }
            $json['plugins'][$key] = array(
                'name' => $single_plugin['Name'],
                'version' => $single_plugin['Version'],
                'active' => $active,
                'update' => $update,
            );
        }

        // Checking for themes updated

        wp_update_themes();
        $update_themes = get_site_transient('update_themes');

        $active_theme = get_option('stylesheet');
        foreach ($update_themes->checked as $theme_name => $theme_version) {
            $json['themes'][$theme_name]['name'] = $theme_name;
            $json['themes'][$theme_name]['version'] = $theme_version;
            if (isset($update_themes->response[$theme_name])) {
                $json['themes'][$theme_name]['update']
                = $update_themes->response[$theme_name]['new_version'];
            } else {
                $json['themes'][$theme_name]['update'] = false;
            }
            if ($theme_name == $active_theme) {
                $json['themes'][$theme_name]['active'] = true;
            } else {
                $json['themes'][$theme_name]['active'] = false;
            }
        }

        return $json;

    }
	/**
 	* Performs updates on plugins or themes
 	* 
 	* @param array $wprequest gets plugins to be updated
	*
	* @param array $json gets total plugins to be updated
 	*
	* @return null 
	*/
    public function update($wprequest, $json)
    {
        $param = $wprequest->get_params();

        if (isset($param['updateversion'])) {
            $module_to_update = $param['updateversion'];
			// check if plugin supplied exist in list of plugins
			
            if (in_array($module_to_update, array_keys($json['plugins']))
                || in_array($module_to_update, array_keys($json['themes']))
                || $module_to_update == 'update-core.php' || $module_to_update == 'update-db.php'
            ) { // check if update is available in list of plugins
                if ((isset($json['plugins'][$module_to_update])
                    && $json['plugins'][$module_to_update]['update'] == "false")
                ) {

                    return new WP_Error(
                        'not_valid_request',
                        'Plugin is already up to date.',
                        array('status' => 404)
                    );
                } elseif ((isset($json['themes'][$module_to_update])
                    && $json['themes'][$module_to_update]['update'] == false)) {
                    return new WP_Error(
                        'not_valid_request',
                        'Theme is already up to date.',
                        array('status' => 404)
                    );
                }
                if (isset($json['themes'][$module_to_update])) {
					// update theme 
                   return $this->updateThemes($json, $module_to_update);
                }
                if ($module_to_update == 'update-db.php') {
                    // update db
                    define('WP_INSTALLING', true);
                    nocache_headers();
                    require_once ABSPATH . 'wp-admin/includes/upgrade.php';
                    delete_site_transient('update_core');
                    wp_upgrade();
                    return;
                } elseif ($module_to_update == 'update-core.php') {

                    return $this->updateCore($json, $module_to_update);
                } else {
                    return $this->updateModules($json, $module_to_update);
                }
            } else {
				// if plugin supplied is not found
                return new WP_Error(
                    'plugin_not_found',
                    'Plugin not found.',
                    array('status' => 500)
                );
            }
        }
    }
	/**
 	* Performs updates on themes
 	*
	*
	* @param array $json gets total plugins to be updated
	*
	* @param array $module_to_update gets current plugin to be updated
	*
 	* @return string  
	*/
    public function updateThemes($json, $module_to_update){
        $theme = $module_to_update;
		// get core files to run upgrader
        require_once ABSPATH . 'wp-admin/includes/class-wp-upgrader.php';
        include_once ABSPATH . 'wp-admin/includes/file.php';
        include_once ABSPATH . 'wp-admin/includes/misc.php';
        define('FS_METHOD', 'direct');
		// upgrade themes
        $upgrader = new Theme_Upgrader();
        $upgrader->upgrade($theme);

        $response = array('success' => true, 'data' => ['Theme updated.']);
        return $response;
    }
	
	/**
 	* Performs updates on plugins
 	* 
 	* @param array $json gets total plugins to be updated
	*
	* @param array $module_to_update gets current plugin to be updated
	*
 	* @return string  
	*/
    public function updateModules($json, $module_to_update){
        ob_start();
		// get core files to run upgrader
        include_once ABSPATH . 'wp-admin/includes/class-wp-upgrader.php';
        include_once ABSPATH . 'wp-admin/includes/file.php';
        include_once ABSPATH . 'wp-admin/includes/plugin.php';
        include_once ABSPATH . 'wp-admin/includes/misc.php';

        define('FS_METHOD', 'direct');
		// get upgrade plugins
        add_filter( 'wp_doing_cron', '__return_true' );
        $upgrader = new Plugin_Upgrader();
        $upgraded = $upgrader->upgrade($module_to_update);
		remove_filter( 'wp_doing_cron', '__return_true' );
        ob_get_clean();

        $url = get_site_url() . "/wp-json/pedalo/plugin_session";
         $headers = array(
            "Content-Type"=>"application/json",
            "Accept"=>"application/json; charset=UTF-8",
            "PEDALO-API-KEY"=>$_SERVER['HTTP_PEDALO_API_KEY'],
            "Authorization"=>"Basic cGVkYWxvOlBlZGFsbzEyMyE=",
        );
        $body = array('updateversion' =>'update-db.php');
        $args = array(                  
                  'timeout' => 60,
                  'redirection' => 5,
                  'blocking' => true,
                  'headers'  => $headers,
                  'sslverify' => false,
                  'body'     => json_encode($body),
                  'cookies'  => array()
                );

        $res = wp_remote_post($url, $args );
        
        $response = array('success' => true,
            'data' => ['plugin updated.']);
        return $response;
    }
	/**
 	* Performs updates on core
 	* 
 	* @param array $json gets total plugins to be updated
	*
	* @param array $module_to_update gets current plugin to be updated
	*
 	* @return string  
	*/

    public function updateCore($json, $module_to_update)
    {
		// check if core is already updated
        if ($json['core']['update'] == false) {
            return new WP_Error(
                'core_already_updated',
                'Core is already updated.',
                array('status' => 500));
        }

        ob_start();
        global $wp_filesystem;
        $reinstall = false;
        include_once ABSPATH . 'wp-admin/includes/class-wp-upgrader.php';
        include_once ABSPATH . 'wp-admin/includes/update.php';
        include_once ABSPATH . 'wp-admin/includes/file.php';
        include_once ABSPATH . 'wp-admin/includes/misc.php';

        define('FS_METHOD', 'direct');
        if ($reinstall) {
            $url = 'update-core.php?action=do-core-reinstall';
        } else {
            $url = 'update-core.php?action=do-core-upgrade';
        }
        $url = wp_nonce_url($url, 'upgrade-core');

        $version = isset($_POST['version']) ? sanitize_text_field($_POST['version']) : false;
        $locale = isset($_POST['locale']) ? sanitize_text_field($_POST['locale']) : sanitize_text_field('en_US');
        $update = find_core_update($json['core']['update'], $locale);

        if (!$update) {
            return;
        }

        // Allow relaxed file ownership writes for
        // User-initiated upgrades when the API specifies
        // that it's safe to do so.
        // This only happens when there are no new files to create.
        $allow_relaxed_file_ownership = !$reinstall && isset($update->new_files) && !$update->new_files;

        add_filter('update_feedback', 'show_message');

        $upgrader = new Core_Upgrader();
        $result = $upgrader->upgrade(
            $update
        );

        if (is_wp_error($result)) {
            show_message($result);
            if ('up_to_date' != $result->get_error_code()
                && 'locked' != $result->get_error_code()
            ) {
                show_message(__('Installation Failed'));
            }
            return;
        }

        $url = get_site_url() . "/wp-json/pedalo/plugin_session";
         $headers = array(
            "Content-Type"=>"application/json",
            "Accept"=>"application/json; charset=UTF-8",
            "PEDALO-API-KEY"=>$_SERVER['HTTP_PEDALO_API_KEY'],
            "Authorization"=>"Basic cGVkYWxvOlBlZGFsbzEyMyE=",
        );
        $body = array('updateversion' =>'update-db.php');
		$args = array(                  
                  'timeout' => 60,
                  'redirection' => 5,
                  'blocking' => true,
                  'headers'  => $headers,
			      'sslverify' => false,
                  'body'     => json_encode($body),
                  'cookies'  => array()
                );

        $res = wp_remote_post($url, $args );
        //show_message(__('WordPress updated successfully'));

        ob_end_clean();

        $response = array('success' => true, 'data' => ['core updated.']);
        return $response;
    }
}
